package org.sonatype.ahc.certs.demo;

/*
 * Copyright (c) 2010 Sonatype, Inc. All rights reserved.
 *
 * This program is licensed to you under the Apache License Version 2.0, 
 * and you may not use this file except in compliance with the Apache License Version 2.0. 
 * You may obtain a copy of the Apache License Version 2.0 at http://www.apache.org/licenses/LICENSE-2.0.
 *
 * Unless required by applicable law or agreed to in writing, 
 * software distributed under the Apache License Version 2.0 is distributed on an 
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the Apache License Version 2.0 for the specific language governing permissions and limitations there under.
 */

import java.io.File;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.jetty.http.security.Constraint;
import org.sonatype.ahc.certs.AHCKey;
import org.sonatype.ahc.certs.CertificateManager;
import org.sonatype.ahc.certs.Certificates;
import org.sonatype.ahc.certs.Configurator;
import org.sonatype.ahc.certs.Connector;
import org.sonatype.ahc.certs.DefaultConfigurator;
import org.sonatype.ahc.certs.DefaultConnector;
import org.sonatype.ahc.certs.Requests;
import org.sonatype.ahc.certs.extractor.DefaultCertificateExtractorUtility;
import org.sonatype.ahc.certs.manager.DefaultCertificateManager;
import org.sonatype.tests.http.server.jetty.behaviour.filesystem.Get;
import org.sonatype.tests.http.server.jetty.impl.JettyServerProvider;
import org.sonatype.tests.http.server.jetty.impl.JettyServerProvider.CertificateHolder;

import com.ning.http.client.Response;

/**
 * @author Benjamin Hanzelmann
 *
 */
public class Demo2
{
    public static void main( String[] args )
        throws Exception
    {
        JettyServerProvider server = setupServer();

        Response response;
        try
        {
            CertificateManager manager = extractFile( "src/main/resources/tinyca2_client-cert.p12", "client" );

            Configurator configurator = new DefaultConfigurator( manager );

            Connector connector = new DefaultConnector( configurator );
            
            AHCKey key = (AHCKey) manager.getEntry( "client" );
            addUser( server, key );

            response = connector.execute( Requests.prepareGet( server.getUrl() + "/test.txt" ) ).get();
        }
        finally
        {
            server.stop();
        }
        System.err.println( response.getResponseBody() );
    }

    /**
     * @param server
     * @param key
     */
    private static void addUser( JettyServerProvider server, AHCKey key )
    {
        CertificateHolder holder =
            new JettyServerProvider.CertificateHolder( Certificates.unpackCertificates( key.getCertificateChain() ) );
        server.addUser( key.getAlias(), holder );
    }

    private static JettyServerProvider setupServer()
        throws Exception
    {
        JettyServerProvider server = new JettyServerProvider();
        server.setSSL( "src/main/resources/tinyca2_server.jks", "server" );
        server.addAuthentication( "/*", Constraint.__CERT_AUTH2 );
        server.addBehaviour( "/*", new Get( "src/main/resources/demo2" ) );
        server.start();
        return server;
    }

    private static CertificateManager extractFile( String path, String password )
    {
        File file = new File( path );

        Map<String, Object> hints = new HashMap<String, Object>();
        hints.put( "password", password );

        CertificateManager certManager = new DefaultCertificateManager();

        certManager.add( new DefaultCertificateExtractorUtility().extract( file, hints ) );
        return certManager;
    }

}
